@@ -1,7 +1,7 @@
-/*******************************************************************
+/********************************************************************
  * This file is part of the Emulex Linux Device Driver for         *
  * Fibre Channel Host Bus Adapters.                                *
- * Copyright (C) 2003-2006 Emulex.  All rights reserved.           *
+ * Copyright (C) 2003-2007 Emulex.  All rights reserved.           *
  * EMULEX and SLI are trademarks of Emulex.                        *
  * www.emulex.com                                                  *
  *                                                                 *
@@ -19,7 +19,7 @@
  *******************************************************************/
 
 /*
- * $Id: lpfc_fcp.c 2905 2006-04-13 17:11:39Z sf_support $
+ * $Id: lpfc_fcp.c 3020 2007-02-28 21:23:36Z sf_support $
  */
 
 #include <linux/version.h>
@@ -1101,6 +1101,103 @@ static CLASS_DEVICE_ATTR(disc_plogi, S_IRUGO, lpfc_disc_plogi_show, NULL);
 static CLASS_DEVICE_ATTR(disc_unused, S_IRUGO, lpfc_disc_unused_show, NULL);
 static CLASS_DEVICE_ATTR(outfcpio, S_IRUGO, lpfc_outfcpio_show, NULL);
 
+static char *lpfc_soft_wwpn_key = "C99G71SL8032A";
+
+static ssize_t
+lpfc_soft_wwpn_enable_store(struct class_device *cdev, const char *buf,
+				size_t count)
+{
+	struct Scsi_Host *host = class_to_shost(cdev);
+	struct lpfc_hba *phba = (struct lpfc_hba*)host->hostdata[0];
+	unsigned int cnt = count;
+
+	/*
+	 * We're doing a simple sanity check for soft_wwpn setting.
+	 * We require that the user write a specific key to enable
+	 * the soft_wwpn attribute to be settable. Once the attribute
+	 * is written, the enable key resets. If further updates are
+	 * desired, the key must be written again to re-enable the
+	 * attribute.
+	 *
+	 * The "key" is not secret - it is a hardcoded string shown
+	 * here. The intent is to protect against the random user or
+	 * application that is just writing attributes.
+	 */
+
+	/* count may include a LF at end of string */
+	if (buf[cnt-1] == '\n')
+		cnt--;
+
+	if ((cnt != strlen(lpfc_soft_wwpn_key)) ||
+	    (strncmp(buf, lpfc_soft_wwpn_key, strlen(lpfc_soft_wwpn_key)) != 0))
+		return -EINVAL;
+
+	phba->soft_wwpn_enable = 1;
+	return count;
+}
+static CLASS_DEVICE_ATTR(lpfc_soft_wwpn_enable, S_IWUSR, NULL,
+				lpfc_soft_wwpn_enable_store);
+
+static ssize_t
+lpfc_soft_wwpn_show(struct class_device *cdev, char *buf) {
+	struct Scsi_Host *host = class_to_shost(cdev);
+	struct lpfc_hba *phba = (struct lpfc_hba*)host->hostdata[0];
+	return snprintf(buf, PAGE_SIZE, "0x%llx\n", 
+			(unsigned long long)phba->cfg_soft_wwpn); }
+
+
+static ssize_t
+lpfc_soft_wwpn_store(struct class_device *cdev, const char *buf, size_t count)
+{
+	struct Scsi_Host *host = class_to_shost(cdev);
+	struct lpfc_hba *phba = (struct lpfc_hba*)host->hostdata[0];
+	unsigned int i, j, cnt=count;
+	u8 wwpn[8];
+
+	/* count may include a LF at end of string */
+	if (buf[cnt-1] == '\n')
+		cnt--;
+
+	if (!phba->soft_wwpn_enable || (cnt < 16) || (cnt > 18) ||
+	    ((cnt == 17) && (*buf++ != 'x')) ||
+	    ((cnt == 18) && ((*buf++ != '0') || (*buf++ != 'x'))))
+		return -EINVAL;
+
+	phba->soft_wwpn_enable = 0;
+
+	memset(wwpn, 0, sizeof(wwpn));
+
+	/* Validate and store the new name */
+	for (i=0, j=0; i < 16; i++) {
+		if ((*buf >= 'a') && (*buf <= 'f'))
+			j = ((j << 4) | ((*buf++ -'a') + 10));
+		else if ((*buf >= 'A') && (*buf <= 'F'))
+			j = ((j << 4) | ((*buf++ -'A') + 10));
+		else if ((*buf >= '0') && (*buf <= '9'))
+			j = ((j << 4) | (*buf++ -'0'));
+		else
+			return -EINVAL;
+		if (i % 2) {
+			wwpn[i/2] = j & 0xff;
+			j = 0;
+		}
+	}
+	phba->cfg_soft_wwpn = lpfc_wwn_to_u64(wwpn);
+
+	dev_printk(KERN_NOTICE, &phba->pcidev->dev,
+		   "lpfc%d: Reinitializing to use soft_wwpn\n", phba->brd_no);
+
+	lpfc_offline(phba);
+	lpfc_sli_brdrestart(phba);
+	lpfc_online(phba);
+
+	return count;
+}
+static CLASS_DEVICE_ATTR(lpfc_soft_wwpn, S_IRUGO | S_IWUSR,\
+			 lpfc_soft_wwpn_show, lpfc_soft_wwpn_store);
+
+
+
 /*
 # lpfc_log_verbose: Only turn this flag on if you are willing to risk being
 # deluged with LOTS of information.
@@ -1111,7 +1208,6 @@ static CLASS_DEVICE_ATTR(outfcpio, S_IRUGO, lpfc_outfcpio_show, NULL);
 # LOG_MBOX                      0x4        Mailbox events
 # LOG_INIT                      0x8        Initialization events
 # LOG_LINK_EVENT                0x10       Link events
-# LOG_IP                        0x20       IP traffic history
 # LOG_FCP                       0x40       FCP traffic history
 # LOG_NODE                      0x80       Node table events
 # LOG_MISC                      0x400      Miscellaneous events
@@ -1165,9 +1261,19 @@ LPFC_ATTR_R(scan_down, 1, 0, 1,
 # until the timer expires. Value range is [0,255]. Default value is 20.
 # NOTE: this MUST be less then the SCSI Layer command timeout - 1.
 */
-LPFC_ATTR_RW(nodev_tmo, 30, 0, 255,
+LPFC_ATTR_RW(nodev_tmo, 20, 0, 255,
 	     "Seconds driver will hold I/O waiting for a device to come back");
 
+/*
+# lpfc_linkdown_tmo: If set, will hold all I/O errors on devices that disappear,
+# after a link down event, until the timer expires. Value range is [0,254].
+# MUST be less then lpfc_nodev_tmo if set. Default value is 0.
+# If not set, lpfc_nodev_tmo is used.
+# NOTE: this MUST be less then the SCSI Layer command timeout - 1.
+*/
+LPFC_ATTR_RW(linkdown_tmo, 0, 0, 254,
+	     "Seconds driver will hold I/O waiting for a device to come back after a link down");
+
 /*
 # lpfc_topology:  link topology for init link
 #            0x0  = attempt loop mode then point-to-point
@@ -1245,6 +1351,22 @@ LPFC_ATTR(multi_ring_support, LPFC_1_PRIMARY_RING, LPFC_1_PRIMARY_RING,
 	     LPFC_2_PRIMARY_RING, "Determines number of primary SLI rings to "
 	     "spread IOCB entries across");
 
+/*
+# lpfc_multi_ring_rctl:  If lpfc_multi_ring_support is enabled, this
+# identifies what rctl value to configure the additional ring for.
+# Value range is [1,0xff]. Default value is 4 (Unsolicated Data).
+*/
+LPFC_ATTR(multi_ring_rctl, FC_UNSOL_DATA, 1,
+	     255, "Identifies RCTL for additional ring configuration");
+
+/*
+# lpfc_multi_ring_type:  If lpfc_multi_ring_support is enabled, this
+# identifies what type value to configure the additional ring for.
+# Value range is [1,0xff]. Default value is 5 (LLC/SNAP).
+*/
+LPFC_ATTR(multi_ring_type, FC_LLC_SNAP, 1,
+	     255, "Identifies TYPE for additional ring configuration");
+
 /*
 # lpfc_fdmi_on: controls FDMI support.
 #       0 = no FDMI support
@@ -1302,6 +1424,54 @@ LPFC_ATTR_RW(discovery_wait_limit, CFG_DISC_INFINITE_WAIT, 0,
 	     "The maximum number of seconds driver waits for the discovery "
 	     "to complete");
 
+/*
+# lpfc_pci_max_read:  Maximum DMA read byte count. This parameter can have
+# values 512, 1024, 2048, 4096. Default value is 2048.
+*/
+static int lpfc_pci_max_read = 2048;
+module_param(lpfc_pci_max_read, int, 0);
+MODULE_PARM_DESC(lpfc_pci_max_read,
+                 "Maximum DMA read byte count. Allowed values:"
+			" 512,1024,2048,4096." );
+static ssize_t
+lpfc_pci_max_read_show(struct class_device *cdev, char *buf)
+{
+	struct Scsi_Host *host = class_to_shost(cdev);
+	struct lpfc_hba *phba = (struct lpfc_hba*)host->hostdata[0];
+	uint32_t val = 0;
+	val = phba->cfg_pci_max_read;
+	return snprintf(buf, PAGE_SIZE, "%d\n", val);
+}
+
+static int
+lpfc_pci_max_read_init(struct lpfc_hba *phba, int val)
+{
+	phba->cfg_pci_max_read = 2048;
+	if ((val == 512) || (val == 1024) || (val == 2048)
+		|| (val == 4096))
+		phba->cfg_pci_max_read = val;
+	return 0;
+}
+static int
+lpfc_pci_max_read_set(struct lpfc_hba *phba, int val)
+{
+	uint32_t prev_val;
+	int ret;
+
+	prev_val = phba->cfg_pci_max_read;
+	phba->cfg_pci_max_read = val;
+	if ((ret = lpfc_sli_set_dma_length(phba, 0))) {
+		phba->cfg_pci_max_read = prev_val;
+		return ret;
+	} else
+		return 0;
+}
+
+lpfc_param_store(pci_max_read)
+
+static CLASS_DEVICE_ATTR(lpfc_pci_max_read, S_IRUGO | S_IWUSR,
+                         lpfc_pci_max_read_show, lpfc_pci_max_read_store);
+
 static ssize_t
 sysfs_ctlreg_write(struct kobject *kobj, char *buf, loff_t off, size_t count)
 {
@@ -1670,8 +1840,14 @@ lpfc_get_starget_loss_tmo(struct scsi_target *starget)
 	/*
 	 * Return the driver's global value for device loss timeout plus
 	 * five seconds to allow the driver's nodev timer to run.
+	 * This should be lpfc_nodev_tmo or lpfc_linkdown_tmo, which ever is
+	 * greater.
 	 */
 	fc_starget_dev_loss_tmo(starget) = lpfc_nodev_tmo + 5;
+
+	/* Sanity check the value of lpfc_linkdown_tmo */
+	if (lpfc_nodev_tmo < lpfc_linkdown_tmo)
+		lpfc_linkdown_tmo = 0;
 }
 
 static void
@@ -1681,10 +1857,15 @@ lpfc_set_starget_loss_tmo(struct scsi_target *starget, uint32_t timeout)
 	 * The driver doesn't have a per-target timeout setting.  Set
 	 * this value globally. Keep lpfc_nodev_tmo >= 1.
 	 */
-	if (timeout)
+	if (timeout) {
 		lpfc_nodev_tmo = timeout;
-	else
+		if (timeout < lpfc_linkdown_tmo)
+			lpfc_linkdown_tmo = timeout - 1;
+	}
+	else {
 		lpfc_nodev_tmo = 1;
+		lpfc_linkdown_tmo = 0;
+	}
 }
 
 #ifdef RHEL_U3_FC_XPORT
@@ -1885,6 +2066,10 @@ lpfc_slave_configure(struct scsi_device *sdev)
 		 */
 		target->starget = sdev->sdev_target;
 		fc_starget_dev_loss_tmo(target->starget) = lpfc_nodev_tmo + 5;
+
+		/* Sanity check the value of lpfc_linkdown_tmo */
+		if (lpfc_nodev_tmo < lpfc_linkdown_tmo)
+			lpfc_linkdown_tmo = 0;
 	}
 #endif /* RHEL_FC */
 
@@ -1910,6 +2095,10 @@ lpfc_slave_destroy(struct scsi_device *sdev)
 					kfree(target);
 					phba->device_queue_hash[i] = NULL;
 				}
+#ifdef RHEL_FC
+				if (!target->slavecnt)
+					target->starget = NULL;
+#endif
 				sdev->hostdata = NULL;
 				return;
 			}
@@ -1954,7 +2143,9 @@ static struct class_device_attribute *lpfc_host_attrs[] = {
 	&class_device_attr_lpfc_log_verbose,
 	&class_device_attr_lpfc_lun_queue_depth,
 	&class_device_attr_lpfc_hba_queue_depth,
+	&class_device_attr_lpfc_pci_max_read,
 	&class_device_attr_lpfc_nodev_tmo,
+	&class_device_attr_lpfc_linkdown_tmo,
 	&class_device_attr_lpfc_fcp_class,
 	&class_device_attr_lpfc_use_adisc,
 	&class_device_attr_lpfc_ack0,
@@ -1980,6 +2171,8 @@ static struct class_device_attribute *lpfc_host_attrs[] = {
 	&class_device_attr_lpfc_linkup_wait_limit,
 	&class_device_attr_lpfc_discovery_min_wait,
 	&class_device_attr_lpfc_discovery_wait_limit,
+	&class_device_attr_lpfc_soft_wwpn,
+	&class_device_attr_lpfc_soft_wwpn_enable,
 	NULL,
 };
 
@@ -2084,8 +2277,8 @@ lpfc_extra_ring_setup( struct lpfc_hba *phba)
 	pring->iotag_max = 4096;
 	pring->num_mask = 1;
 	pring->prt[0].profile = 0;	/* Mask 0 */
-	pring->prt[0].rctl = FC_UNSOL_DATA;
-	pring->prt[0].type = 5;
+	pring->prt[0].rctl = phba->cfg_multi_ring_rctl;
+	pring->prt[0].type = phba->cfg_multi_ring_type;
 	pring->prt[0].lpfc_sli_rcv_unsol_event = NULL;
 	return 0;
 }
@@ -2100,7 +2293,7 @@ lpfc_sli_setup(struct lpfc_hba * phba)
 	psli->sliinit.num_rings = MAX_CONFIGURED_RINGS;
 	psli->fcp_ring = LPFC_FCP_RING;
 	psli->next_ring = LPFC_FCP_NEXT_RING;
-	psli->ip_ring = LPFC_IP_RING;
+	psli->extra_ring = LPFC_EXTRA_RING;
 
 	for (i = 0; i < psli->sliinit.num_rings; i++) {
 		pring = &psli->sliinit.ringinit[i];
@@ -2119,7 +2312,7 @@ lpfc_sli_setup(struct lpfc_hba * phba)
 			pring->fast_iotag = pring->iotag_max;
 			pring->num_mask = 0;
 			break;
-		case LPFC_IP_RING:	/* ring 1 - IP */
+		case LPFC_EXTRA_RING:	/* ring 1 - Other protocols */
 			/* numCiocb and numRiocb are used in config_port */
 			pring->numCiocb = SLI2_IOCB_CMD_R1_ENTRIES;
 			pring->numRiocb = SLI2_IOCB_RSP_R1_ENTRIES;
@@ -2174,7 +2367,7 @@ lpfc_sli_setup(struct lpfc_hba * phba)
 				phba->brd_no, totiocb, MAX_SLI2_IOCB);
 	}
 
-	if (lpfc_multi_ring_support == LPFC_2_PRIMARY_RING)
+	if (phba->cfg_multi_ring_support == LPFC_2_PRIMARY_RING)
 		lpfc_extra_ring_setup(phba);
 
 	psli->sliinit.sli_flag = 0;
@@ -2218,6 +2411,8 @@ lpfc_get_cfgparam(struct lpfc_hba *phba)
 	lpfc_cr_delay_init(phba, lpfc_cr_delay);
 	lpfc_cr_count_init(phba, lpfc_cr_count);
 	lpfc_multi_ring_support_init(phba, lpfc_multi_ring_support);
+	lpfc_multi_ring_rctl_init(phba, lpfc_multi_ring_rctl);
+	lpfc_multi_ring_type_init(phba, lpfc_multi_ring_type);
 	lpfc_lun_queue_depth_init(phba, lpfc_lun_queue_depth);
 	lpfc_fcp_class_init(phba, lpfc_fcp_class);
 	lpfc_use_adisc_init(phba, lpfc_use_adisc);
@@ -2225,6 +2420,8 @@ lpfc_get_cfgparam(struct lpfc_hba *phba)
 	lpfc_topology_init(phba, lpfc_topology);
 	lpfc_scan_down_init(phba, lpfc_scan_down);
 	lpfc_nodev_tmo_init(phba, lpfc_nodev_tmo);
+	lpfc_linkdown_tmo_init(phba, lpfc_linkdown_tmo);
+	lpfc_pci_max_read_init(phba, lpfc_pci_max_read);
 	lpfc_link_speed_init(phba, lpfc_link_speed);
 	lpfc_fdmi_on_init(phba, lpfc_fdmi_on);
 	lpfc_discovery_threads_init(phba, lpfc_discovery_threads);
@@ -2233,6 +2430,7 @@ lpfc_get_cfgparam(struct lpfc_hba *phba)
 	lpfc_discovery_min_wait_init(phba, lpfc_discovery_min_wait);
 	lpfc_discovery_wait_limit_init(phba, lpfc_discovery_wait_limit);
 	phba->cfg_scsi_hotplug = 0;
+	phba->cfg_soft_wwpn = 0L;
 
 	switch (phba->pcidev->device) {
 	case PCI_DEVICE_ID_LP101:
@@ -2335,6 +2533,9 @@ lpfc_pci_probe_one(struct pci_dev *pdev, const struct pci_device_id *pid)
 	init_timer(&phba->fc_scantmo);
 	phba->fc_scantmo.function = lpfc_scan_timeout;
 	phba->fc_scantmo.data = (unsigned long)phba;
+	init_timer(&phba->fc_lnkdwntmo);
+	phba->fc_lnkdwntmo.function = lpfc_linkdown_timeout;
+	phba->fc_lnkdwntmo.data = (unsigned long)phba;
 
 	init_timer(&phba->fc_fdmitmo);
 	phba->fc_fdmitmo.function = lpfc_fdmi_tmo;
@@ -2347,6 +2548,11 @@ lpfc_pci_probe_one(struct pci_dev *pdev, const struct pci_device_id *pid)
 	psli->mbox_tmo.function = lpfc_mbox_timeout;
 	psli->mbox_tmo.data = (unsigned long)phba;
 
+	/* This timer is to check for un processed host attention */
+	init_timer(&phba->hatt_tmo);
+	phba->hatt_tmo.function = lpfc_hatt_timeout;
+	phba->hatt_tmo.data = (unsigned long)phba;
+
 	/* Assign an unused board number */
  	phba->brd_no = lpfc_get_brd_no(phba);
 	host->unique_id = phba->brd_no;
@@ -2493,6 +2699,7 @@ lpfc_pci_probe_one(struct pci_dev *pdev, const struct pci_device_id *pid)
 	sysfs_create_bin_file(&host->shost_classdev.kobj, &sysfs_mbox_attr);
 	scsi_scan_host(host);
 	phba->fc_flag &= ~FC_LOADING;
+	mod_timer(&phba->hatt_tmo, jiffies + HZ/10);
 	return 0;
 
 out_hba_down:
