@@ -1,51 +1,48 @@
 /*******************************************************************
  * This file is part of the Emulex Linux Device Driver for         *
  * Fibre Channel Host Bus Adapters.                                *
- * Refer to the README file included with this package for         *
- * driver version and adapter support.                             *
  * Copyright (C) 2003-2005 Emulex.  All rights reserved.           *
+ * EMULEX and SLI are trademarks of Emulex.                        *
  * www.emulex.com                                                  *
  *                                                                 *
  * This program is free software; you can redistribute it and/or   *
- * modify it under the terms of the GNU General Public License     *
- * as published by the Free Software Foundation; either version 2  *
- * of the License, or (at your option) any later version.          *
- *                                                                 *
- * This program is distributed in the hope that it will be useful, *
- * but WITHOUT ANY WARRANTY; without even the implied warranty of  *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the   *
- * GNU General Public License for more details, a copy of which    *
- * can be found in the file COPYING included with this package.    *
+ * modify it under the terms of version 2 of the GNU General       *
+ * Public License as published by the Free Software Foundation.    *
+ * This program is distributed in the hope that it will be useful. *
+ * ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND          *
+ * WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,  *
+ * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT, ARE      *
+ * DISCLAIMED, EXCEPT TO THE EXTENT THAT SUCH DISCLAIMERS ARE HELD *
+ * TO BE LEGALLY INVALID.  See the GNU General Public License for  *
+ * more details, a copy of which can be found in the file COPYING  *
+ * included with this package.                                     *
  *******************************************************************/
 
 /*
- * $Id: lpfc_mbox.c 328 2005-05-03 15:20:43Z sf_support $
+ * $Id: lpfc_mbox.c 2970 2006-10-25 13:07:37Z sf_support $
  */
 #include <linux/version.h>
-#include <linux/spinlock.h>
+#include <linux/blkdev.h>
+#include <linux/dma-mapping.h>
 #include <linux/pci.h>
-
-
-#include <linux/blk.h>
-#include <scsi.h>
-
-#include "lpfc_hw.h"
+#include <linux/spinlock.h>
+#include <scsi/scsi_device.h>
 #include "lpfc_sli.h"
-#include "lpfc_mem.h"
-#include "lpfc_sched.h"
 #include "lpfc_disc.h"
-#include "lpfc.h"
-#include "lpfc_logmsg.h"
 #include "lpfc_scsi.h"
+#include "lpfc.h"
 #include "lpfc_crtn.h"
-#include "lpfc_cfgparm.h"
+#include "lpfc_hw.h"
+#include "lpfc_logmsg.h"
+#include "lpfc_mem.h"
+#include "lpfc_compat.h"
 
 /**********************************************/
 
 /*                mailbox command             */
 /**********************************************/
 void
-lpfc_dump_mem(lpfcHBA_t * phba, LPFC_MBOXQ_t * pmb, uint16_t offset)
+lpfc_dump_mem(struct lpfc_hba * phba, LPFC_MBOXQ_t * pmb, uint16_t offset)
 {
 	MAILBOX_t *mb;
 	void *ctx;
@@ -61,11 +58,9 @@ lpfc_dump_mem(lpfcHBA_t * phba, LPFC_MBOXQ_t * pmb, uint16_t offset)
 	mb->un.varDmp.entry_index = offset;
 	mb->un.varDmp.region_id = DMP_REGION_VPD;
 	mb->un.varDmp.word_cnt = (DMP_RSP_SIZE / sizeof (uint32_t));
-
 	mb->un.varDmp.co = 0;
 	mb->un.varDmp.resp_offset = 0;
 	pmb->context2 = ctx;
-
 	mb->mbxOwner = OWN_HOST;
 	return;
 }
@@ -75,7 +70,7 @@ lpfc_dump_mem(lpfcHBA_t * phba, LPFC_MBOXQ_t * pmb, uint16_t offset)
 /*                mailbox command             */
 /**********************************************/
 void
-lpfc_read_nv(lpfcHBA_t * phba, LPFC_MBOXQ_t * pmb)
+lpfc_read_nv(struct lpfc_hba * phba, LPFC_MBOXQ_t * pmb)
 {
 	MAILBOX_t *mb;
 
@@ -91,25 +86,28 @@ lpfc_read_nv(lpfcHBA_t * phba, LPFC_MBOXQ_t * pmb)
 /*                mailbox command             */
 /**********************************************/
 int
-lpfc_read_la(lpfcHBA_t * phba, LPFC_MBOXQ_t * pmb)
+lpfc_read_la(struct lpfc_hba * phba, LPFC_MBOXQ_t * pmb)
 {
 	MAILBOX_t *mb;
-	DMABUF_t *mp;
-	LPFC_SLI_t *psli;
+	struct lpfc_dmabuf *mp;
+	struct lpfc_sli *psli;
 
 	psli = &phba->sli;
 	mb = &pmb->mb;
 	memset(pmb, 0, sizeof (LPFC_MBOXQ_t));
 
 	/* Get a buffer to hold the loop map */
-	if (((mp = kmalloc(sizeof (DMABUF_t), GFP_ATOMIC)) == 0) ||
+	if (((mp = kmalloc(sizeof (struct lpfc_dmabuf), GFP_ATOMIC)) == 0) ||
 	    ((mp->virt = lpfc_mbuf_alloc(phba, 0, &(mp->phys))) == 0)) {
 		if (mp)
 			kfree(mp);
 		mb->mbxCommand = MBX_READ_LA64;
 		/* READ_LA: no buffers */
-		lpfc_printf_log(phba->brd_no, &lpfc_msgBlk0300, lpfc_mes0300,
-			       lpfc_msgBlk0300.msgPreambleStr);
+		lpfc_printf_log(phba,
+			       KERN_WARNING,
+			       LOG_MBOX,
+			       "%d:0300 READ_LA: no buffers\n",
+			       phba->brd_no);
 		return (1);
 	}
 	INIT_LIST_HEAD(&mp->list);
@@ -119,7 +117,7 @@ lpfc_read_la(lpfcHBA_t * phba, LPFC_MBOXQ_t * pmb)
 	mb->un.varReadLA.un.lilpBde64.addrLow = putPaddrLow(mp->phys);
 
 	/* Save address for later completion and set the owner to host so that
-	 * the FW knows this mailbox is available for processing. 
+	 * the FW knows this mailbox is available for processing.
 	 */
 	pmb->context1 = (uint8_t *) mp;
 	mb->mbxOwner = OWN_HOST;
@@ -131,7 +129,7 @@ lpfc_read_la(lpfcHBA_t * phba, LPFC_MBOXQ_t * pmb)
 /*                 mailbox command            */
 /**********************************************/
 void
-lpfc_clear_la(lpfcHBA_t * phba, LPFC_MBOXQ_t * pmb)
+lpfc_clear_la(struct lpfc_hba * phba, LPFC_MBOXQ_t * pmb)
 {
 	MAILBOX_t *mb;
 
@@ -149,23 +147,19 @@ lpfc_clear_la(lpfcHBA_t * phba, LPFC_MBOXQ_t * pmb)
 /*                    mailbox command             */
 /**************************************************/
 void
-lpfc_config_link(lpfcHBA_t * phba, LPFC_MBOXQ_t * pmb)
+lpfc_config_link(struct lpfc_hba * phba, LPFC_MBOXQ_t * pmb)
 {
-	MAILBOX_t *mb;
-	lpfcCfgParam_t *clp;
-
-	clp = &phba->config[0];
-	mb = &pmb->mb;
+	MAILBOX_t *mb = &pmb->mb;
 	memset(pmb, 0, sizeof (LPFC_MBOXQ_t));
 
 	/* NEW_FEATURE
-	 * SLI-2, Coalescing Response Feature. 
+	 * SLI-2, Coalescing Response Feature.
 	 */
-	if (clp[LPFC_CFG_CR_DELAY].a_current) {
+	if (phba->cfg_cr_delay) {
 		mb->un.varCfgLnk.cr = 1;
 		mb->un.varCfgLnk.ci = 1;
-		mb->un.varCfgLnk.cr_delay = clp[LPFC_CFG_CR_DELAY].a_current;
-		mb->un.varCfgLnk.cr_count = clp[LPFC_CFG_CR_COUNT].a_current;
+		mb->un.varCfgLnk.cr_delay = phba->cfg_cr_delay;
+		mb->un.varCfgLnk.cr_count = phba->cfg_cr_count;
 	}
 
 	mb->un.varCfgLnk.myId = phba->fc_myDID;
@@ -177,7 +171,7 @@ lpfc_config_link(lpfcHBA_t * phba, LPFC_MBOXQ_t * pmb)
 	mb->un.varCfgLnk.crtov = phba->fc_crtov;
 	mb->un.varCfgLnk.citov = phba->fc_citov;
 
-	if (clp[LPFC_CFG_ACK0].a_current)
+	if (phba->cfg_ack0)
 		mb->un.varCfgLnk.ack0_enable = 1;
 
 	mb->mbxCommand = MBX_CONFIG_LINK;
@@ -190,11 +184,11 @@ lpfc_config_link(lpfcHBA_t * phba, LPFC_MBOXQ_t * pmb)
 /*                  mailbox command           */
 /**********************************************/
 void
-lpfc_init_link(lpfcHBA_t * phba,
+lpfc_init_link(struct lpfc_hba * phba,
 	       LPFC_MBOXQ_t * pmb, uint32_t topology, uint32_t linkspeed)
 {
 	lpfc_vpd_t *vpd;
-	LPFC_SLI_t *psli;
+	struct lpfc_sli *psli;
 	MAILBOX_t *mb;
 
 	mb = &pmb->mb;
@@ -218,20 +212,31 @@ lpfc_init_link(lpfcHBA_t * phba,
 		break;
 	}
 
+	/* Enable asynchronous ABTS responses from firmware */
+	mb->un.varInitLnk.link_flags |= FLAGS_IMED_ABORT;
+
+	/* NEW_FEATURE
+	 * Setting up the link speed
+	 */
 	vpd = &phba->vpd;
-	if (vpd->rev.feaLevelHigh >= 0x02) {
-		switch(linkspeed) {
-		case LINK_SPEED_1G:
-		case LINK_SPEED_2G:
-		case LINK_SPEED_4G:
-			mb->un.varInitLnk.link_flags |= FLAGS_LINK_SPEED;
-			mb->un.varInitLnk.link_speed = linkspeed;
+	if (vpd->rev.feaLevelHigh >= 0x02){
+		switch(linkspeed){
+			case LINK_SPEED_1G:
+			case LINK_SPEED_2G:
+			case LINK_SPEED_4G:
+				mb->un.varInitLnk.link_flags |=
+							FLAGS_LINK_SPEED;
+				mb->un.varInitLnk.link_speed = linkspeed;
+			break;
+			case LINK_SPEED_AUTO:
+			default:
+				mb->un.varInitLnk.link_speed =
+							LINK_SPEED_AUTO;
 			break;
-		case LINK_SPEED_AUTO:
-		default:
-			mb->un.varInitLnk.link_speed = LINK_SPEED_AUTO;
 		}
-	} else
+
+	}
+	else
 		mb->un.varInitLnk.link_speed = LINK_SPEED_AUTO;
 
 	mb->mbxCommand = (volatile uint8_t)MBX_INIT_LINK;
@@ -245,11 +250,11 @@ lpfc_init_link(lpfcHBA_t * phba,
 /*                    mailbox command         */
 /**********************************************/
 int
-lpfc_read_sparam(lpfcHBA_t * phba, LPFC_MBOXQ_t * pmb)
+lpfc_read_sparam(struct lpfc_hba * phba, LPFC_MBOXQ_t * pmb)
 {
-	DMABUF_t *mp;
+	struct lpfc_dmabuf *mp;
 	MAILBOX_t *mb;
-	LPFC_SLI_t *psli;
+	struct lpfc_sli *psli;
 
 	psli = &phba->sli;
 	mb = &pmb->mb;
@@ -259,19 +264,22 @@ lpfc_read_sparam(lpfcHBA_t * phba, LPFC_MBOXQ_t * pmb)
 
 	/* Get a buffer to hold the HBAs Service Parameters */
 
-	if (((mp = kmalloc(sizeof (DMABUF_t), GFP_ATOMIC)) == 0) ||
+	if (((mp = kmalloc(sizeof (struct lpfc_dmabuf), GFP_ATOMIC)) == 0) ||
 	    ((mp->virt = lpfc_mbuf_alloc(phba, 0, &(mp->phys))) == 0)) {
 		if (mp)
 			kfree(mp);
 		mb->mbxCommand = MBX_READ_SPARM64;
 		/* READ_SPARAM: no buffers */
-		lpfc_printf_log(phba->brd_no, &lpfc_msgBlk0301,
-			       lpfc_mes0301, lpfc_msgBlk0301.msgPreambleStr);
+		lpfc_printf_log(phba,
+			        KERN_WARNING,
+			        LOG_MBOX,
+			        "%d:0301 READ_SPARAM: no buffers\n",
+			        phba->brd_no);
 		return (1);
 	}
 	INIT_LIST_HEAD(&mp->list);
 	mb->mbxCommand = MBX_READ_SPARM64;
-	mb->un.varRdSparm.un.sp64.tus.f.bdeSize = sizeof (SERV_PARM);
+	mb->un.varRdSparm.un.sp64.tus.f.bdeSize = sizeof (struct serv_parm);
 	mb->un.varRdSparm.un.sp64.addrHigh = putPaddrHigh(mp->phys);
 	mb->un.varRdSparm.un.sp64.addrLow = putPaddrLow(mp->phys);
 
@@ -281,34 +289,12 @@ lpfc_read_sparam(lpfcHBA_t * phba, LPFC_MBOXQ_t * pmb)
 	return (0);
 }
 
-/**********************************************/
-/*  lpfc_read_rpi    Issue a READ RPI         */
-/*                   mailbox command          */
-/**********************************************/
-int
-lpfc_read_rpi(lpfcHBA_t * phba, uint32_t rpi, LPFC_MBOXQ_t * pmb, uint32_t flag)
-{
-	MAILBOX_t *mb;
-
-	mb = &pmb->mb;
-	memset(pmb, 0, sizeof (LPFC_MBOXQ_t));
-
-	mb->un.varRdRPI.reqRpi = (volatile uint16_t)rpi;
-
-	mb->mbxCommand = MBX_READ_RPI64;
-	mb->mbxOwner = OWN_HOST;
-
-	mb->un.varWords[30] = flag;	/* Set flag to issue action on cmpl */
-
-	return (0);
-}
-
 /********************************************/
 /*  lpfc_unreg_did  Issue a UNREG_DID       */
 /*                  mailbox command         */
 /********************************************/
 void
-lpfc_unreg_did(lpfcHBA_t * phba, uint32_t did, LPFC_MBOXQ_t * pmb)
+lpfc_unreg_did(struct lpfc_hba * phba, uint32_t did, LPFC_MBOXQ_t * pmb)
 {
 	MAILBOX_t *mb;
 
@@ -323,11 +309,11 @@ lpfc_unreg_did(lpfcHBA_t * phba, uint32_t did, LPFC_MBOXQ_t * pmb)
 }
 
 /***********************************************/
-
-/*                  command to write slim      */
+/*  lpfc_set_slim  Issue a SET_SLIM            */
+/*                 command to write slim       */
 /***********************************************/
 void
-lpfc_set_slim(lpfcHBA_t * phba, LPFC_MBOXQ_t * pmb, uint32_t addr,
+lpfc_set_slim(struct lpfc_hba * phba, LPFC_MBOXQ_t * pmb, uint32_t addr,
 	      uint32_t value)
 {
 	MAILBOX_t *mb;
@@ -335,17 +321,8 @@ lpfc_set_slim(lpfcHBA_t * phba, LPFC_MBOXQ_t * pmb, uint32_t addr,
 	mb = &pmb->mb;
 	memset(pmb, 0, sizeof (LPFC_MBOXQ_t));
 
-	/* addr = 0x090597 is AUTO ABTS disable for ELS commands */
-	/* addr = 0x052198 is DELAYED ABTS enable for ELS commands */
-
-	/*
-	 * Always turn on DELAYED ABTS for ELS timeouts 
-	 */
-	if ((addr == 0x052198) && (value == 0))
-		value = 1;
-
-	mb->un.varWords[0] = addr;
-	mb->un.varWords[1] = value;
+	mb->un.varSetSlim.varNumber = addr;
+	mb->un.varSetSlim.varValue = value;
 
 	mb->mbxCommand = MBX_SET_SLIM;
 	mb->mbxOwner = OWN_HOST;
@@ -353,36 +330,11 @@ lpfc_set_slim(lpfcHBA_t * phba, LPFC_MBOXQ_t * pmb, uint32_t addr,
 }
 
 /**********************************************/
-/*  lpfc_config_farp  Issue a CONFIG FARP     */
-/*                    mailbox command         */
-/**********************************************/
-void
-lpfc_config_farp(lpfcHBA_t * phba, LPFC_MBOXQ_t * pmb)
-{
-	MAILBOX_t *mb;
-
-	mb = &pmb->mb;
-	memset(pmb, 0, sizeof (LPFC_MBOXQ_t));
-
-	mb->un.varCfgFarp.filterEnable = 1;
-	mb->un.varCfgFarp.portName = 1;
-	mb->un.varCfgFarp.nodeName = 1;
-
-	memcpy((uint8_t *) & mb->un.varCfgFarp.portname,
-	       (uint8_t *) & phba->fc_portname, sizeof (NAME_TYPE));
-	memcpy((uint8_t *) & mb->un.varCfgFarp.nodename,
-	       (uint8_t *) & phba->fc_portname, sizeof (NAME_TYPE));
-	mb->mbxCommand = MBX_CONFIG_FARP;
-	mb->mbxOwner = OWN_HOST;
-	return;
-}
-
-/**********************************************/
-/*  lpfc_read_nv  Issue a READ CONFIG         */
+/*  lpfc_read_config  Issue a READ CONFIG     */
 /*                mailbox command             */
 /**********************************************/
 void
-lpfc_read_config(lpfcHBA_t * phba, LPFC_MBOXQ_t * pmb)
+lpfc_read_config(struct lpfc_hba * phba, LPFC_MBOXQ_t * pmb)
 {
 	MAILBOX_t *mb;
 
@@ -394,18 +346,35 @@ lpfc_read_config(lpfcHBA_t * phba, LPFC_MBOXQ_t * pmb)
 	return;
 }
 
+/*************************************************/
+/*  lpfc_read_lnk_stat  Issue a READ LINK STATUS */
+/*                mailbox command                */
+/*************************************************/
+void
+lpfc_read_lnk_stat(struct lpfc_hba * phba, LPFC_MBOXQ_t * pmb)
+{
+	MAILBOX_t *mb;
+
+	mb = &pmb->mb;
+	memset(pmb, 0, sizeof (LPFC_MBOXQ_t));
+
+	mb->mbxCommand = MBX_READ_LNK_STAT;
+	mb->mbxOwner = OWN_HOST;
+	return;
+}
+
 /********************************************/
 /*  lpfc_reg_login  Issue a REG_LOGIN       */
 /*                  mailbox command         */
 /********************************************/
 int
-lpfc_reg_login(lpfcHBA_t * phba,
+lpfc_reg_login(struct lpfc_hba * phba,
 	       uint32_t did, uint8_t * param, LPFC_MBOXQ_t * pmb, uint32_t flag)
 {
 	uint8_t *sparam;
-	DMABUF_t *mp;
+	struct lpfc_dmabuf *mp;
 	MAILBOX_t *mb;
-	LPFC_SLI_t *psli;
+	struct lpfc_sli *psli;
 
 	psli = &phba->sli;
 	mb = &pmb->mb;
@@ -418,16 +387,18 @@ lpfc_reg_login(lpfcHBA_t * phba,
 	mb->mbxOwner = OWN_HOST;
 
 	/* Get a buffer to hold NPorts Service Parameters */
-	if (((mp = kmalloc(sizeof (DMABUF_t), GFP_ATOMIC)) == 0) ||
+	if (((mp = kmalloc(sizeof (struct lpfc_dmabuf), GFP_ATOMIC)) == 0) ||
 	    ((mp->virt = lpfc_mbuf_alloc(phba, 0, &(mp->phys))) == 0)) {
 		if (mp)
 			kfree(mp);
 
 		mb->mbxCommand = MBX_REG_LOGIN64;
 		/* REG_LOGIN: no buffers */
-		lpfc_printf_log(phba->brd_no, &lpfc_msgBlk0302,
-			       lpfc_mes0302,
-			       lpfc_msgBlk0302.msgPreambleStr,
+		lpfc_printf_log(phba,
+			       KERN_WARNING,
+			       LOG_MBOX,
+			       "%d:0302 REG_LOGIN: no buffers Data x%x x%x\n",
+			       phba->brd_no,
 			       (uint32_t) did, (uint32_t) flag);
 		return (1);
 	}
@@ -435,13 +406,13 @@ lpfc_reg_login(lpfcHBA_t * phba,
 	sparam = mp->virt;
 
 	/* Copy param's into a new buffer */
-	memcpy(sparam, param, sizeof (SERV_PARM));
+	memcpy(sparam, param, sizeof (struct serv_parm));
 
 	/* save address for completion */
 	pmb->context1 = (uint8_t *) mp;
 
 	mb->mbxCommand = MBX_REG_LOGIN64;
-	mb->un.varRegLogin.un.sp64.tus.f.bdeSize = sizeof (SERV_PARM);
+	mb->un.varRegLogin.un.sp64.tus.f.bdeSize = sizeof (struct serv_parm);
 	mb->un.varRegLogin.un.sp64.addrHigh = putPaddrHigh(mp->phys);
 	mb->un.varRegLogin.un.sp64.addrLow = putPaddrLow(mp->phys);
 
@@ -453,7 +424,7 @@ lpfc_reg_login(lpfcHBA_t * phba,
 /*                    mailbox command         */
 /**********************************************/
 void
-lpfc_unreg_login(lpfcHBA_t * phba, uint32_t rpi, LPFC_MBOXQ_t * pmb)
+lpfc_unreg_login(struct lpfc_hba * phba, uint32_t rpi, LPFC_MBOXQ_t * pmb)
 {
 	MAILBOX_t *mb;
 
@@ -468,29 +439,19 @@ lpfc_unreg_login(lpfcHBA_t * phba, uint32_t rpi, LPFC_MBOXQ_t * pmb)
 	return;
 }
 
-/***********************************************/
-/*  lpfc_config_pcb_setup  Issue a CONFIG_PORT */
-/*                   mailbox command           */
-/***********************************************/
-uint32_t *
-lpfc_config_pcb_setup(lpfcHBA_t * phba)
+static void
+lpfc_config_pcb_setup(struct lpfc_hba * phba)
 {
-	LPFC_SLI_t *psli;
-	LPFC_SLI_RING_t *pring;
+	struct lpfc_sli *psli = &phba->sli;
+	struct lpfc_sli_ring *pring;
+	PCB_t *pcbp = &phba->slim2p->pcb;
 	LPFC_RING_INIT_t *pringinit;
-	PCB_t *pcbp;
-	SLI2_SLIM_t *slim2p_virt;
 	dma_addr_t pdma_addr;
 	uint32_t offset;
 	uint32_t iocbCnt;
 	int i;
 
-	psli = &phba->sli;
-
-	slim2p_virt = ((SLI2_SLIM_t *) phba->slim2p.virt);
-	pcbp = &slim2p_virt->un.slim.pcb;
-	psli->MBhostaddr = (uint32_t *) (&slim2p_virt->un.slim.mbx);
-
+	psli->MBhostaddr = (uint32_t *)&phba->slim2p->mbx;
 	pcbp->maxRing = (psli->sliinit.num_rings - 1);
 
 	iocbCnt = 0;
@@ -506,48 +467,38 @@ lpfc_config_pcb_setup(lpfcHBA_t * phba)
 			pcbp->rdsc[i].rspAddrHigh = 0;
 			pcbp->rdsc[i].cmdAddrLow = 0;
 			pcbp->rdsc[i].rspAddrLow = 0;
-			pring->cmdringaddr = 0;
-			pring->rspringaddr = 0;
+			pring->cmdringaddr = NULL;
+			pring->rspringaddr = NULL;
 			continue;
 		}
 		/* Command ring setup for ring */
 		pring->cmdringaddr =
-		    (void *)&slim2p_virt->un.slim.IOCBs[iocbCnt];
+		    (void *)&phba->slim2p->IOCBs[iocbCnt];
 		pcbp->rdsc[i].cmdEntries = pringinit->numCiocb;
 
-		offset =
-		    (uint8_t *) & slim2p_virt->un.slim.IOCBs[iocbCnt] -
-		    (uint8_t *) slim2p_virt;
-		pdma_addr = phba->slim2p.phys + offset;
+		offset = (uint8_t *)&phba->slim2p->IOCBs[iocbCnt] -
+			 (uint8_t *)phba->slim2p;
+		pdma_addr = phba->slim2p_mapping + offset;
 		pcbp->rdsc[i].cmdAddrHigh = putPaddrHigh(pdma_addr);
 		pcbp->rdsc[i].cmdAddrLow = putPaddrLow(pdma_addr);
 		iocbCnt += pringinit->numCiocb;
 
 		/* Response ring setup for ring */
 		pring->rspringaddr =
-		    (void *) &slim2p_virt->un.slim.IOCBs[iocbCnt];
+		    (void *)&phba->slim2p->IOCBs[iocbCnt];
 
 		pcbp->rdsc[i].rspEntries = pringinit->numRiocb;
-		offset =
-		    (uint8_t *) & slim2p_virt->un.slim.IOCBs[iocbCnt] -
-		    (uint8_t *) slim2p_virt;
-		pdma_addr = phba->slim2p.phys + offset;
+		offset = (uint8_t *)&phba->slim2p->IOCBs[iocbCnt] -
+			 (uint8_t *)phba->slim2p;
+		pdma_addr = phba->slim2p_mapping + offset;
 		pcbp->rdsc[i].rspAddrHigh = putPaddrHigh(pdma_addr);
 		pcbp->rdsc[i].rspAddrLow = putPaddrLow(pdma_addr);
 		iocbCnt += pringinit->numRiocb;
 	}
-
-	/* special handling for LC HBAs */
-	if (lpfc_is_LC_HBA(phba->pcidev->device)) {
-		lpfc_hba_init(phba);
-		return (phba->hbainitEx);
-	} else
-		return (0);
 }
 
-
 void
-lpfc_read_rev(lpfcHBA_t * phba, LPFC_MBOXQ_t * pmb)
+lpfc_read_rev(struct lpfc_hba * phba, LPFC_MBOXQ_t * pmb)
 {
 	MAILBOX_t *mb;
 
@@ -560,14 +511,13 @@ lpfc_read_rev(lpfcHBA_t * phba, LPFC_MBOXQ_t * pmb)
 }
 
 void
-lpfc_config_ring(lpfcHBA_t * phba, int ring, LPFC_MBOXQ_t * pmb)
+lpfc_config_ring(struct lpfc_hba * phba, int ring, LPFC_MBOXQ_t * pmb)
 {
 	int i;
-	MAILBOX_t *mb;
-	LPFC_SLI_t *psli;
+	MAILBOX_t *mb = &pmb->mb;
+	struct lpfc_sli *psli;
 	LPFC_RING_INIT_t *pring;
 
-	mb = &pmb->mb;
 	memset(pmb, 0, sizeof (LPFC_MBOXQ_t));
 
 	mb->un.varCfgRing.ring = ring;
@@ -601,107 +551,121 @@ lpfc_config_ring(lpfcHBA_t * phba, int ring, LPFC_MBOXQ_t * pmb)
 	return;
 }
 
-int
-lpfc_config_port(lpfcHBA_t * phba, LPFC_MBOXQ_t * pmb)
+void
+lpfc_config_port(struct lpfc_hba * phba, LPFC_MBOXQ_t * pmb)
 {
-	LPFC_SLI_t *psli;
-	MAILBOX_t *mb;
-	uint32_t *hbainit;
+	MAILBOX_t *mb = &pmb->mb;
 	dma_addr_t pdma_addr;
-	uint32_t offset;
+	uint32_t bar_low, bar_high;
+	size_t offset;
 	HGP hgp;
 	void *to_slim;
-	uint32_t bar0_config_word, bar1_config_word;
+	int i;
 
-	mb = &pmb->mb;
-	memset(pmb, 0, sizeof (LPFC_MBOXQ_t));
-
-	psli = &phba->sli;
+	memset(pmb, 0, sizeof(LPFC_MBOXQ_t));
 	mb->mbxCommand = MBX_CONFIG_PORT;
 	mb->mbxOwner = OWN_HOST;
 
-	mb->un.varCfgPort.pcbLen = sizeof (PCB_t);
-	offset =
-	    (uint8_t *) (&((SLI2_SLIM_t *) phba->slim2p.virt)->un.slim.pcb) -
-	    (uint8_t *) phba->slim2p.virt;
-	pdma_addr = phba->slim2p.phys + offset;
+	mb->un.varCfgPort.pcbLen = sizeof(PCB_t);
+	offset = (uint8_t *)&phba->slim2p->pcb - (uint8_t *)phba->slim2p;
+	pdma_addr = phba->slim2p_mapping + offset;
 	mb->un.varCfgPort.pcbLow = putPaddrLow(pdma_addr);
 	mb->un.varCfgPort.pcbHigh = putPaddrHigh(pdma_addr);
 
 	/* Now setup pcb */
-	((SLI2_SLIM_t *) phba->slim2p.virt)->un.slim.pcb.type =
-	    TYPE_NATIVE_SLI2;
-	((SLI2_SLIM_t *) phba->slim2p.virt)->un.slim.pcb.feature =
-	    FEATURE_INITIAL_SLI2;
+	phba->slim2p->pcb.type = TYPE_NATIVE_SLI2;
+	phba->slim2p->pcb.feature = FEATURE_INITIAL_SLI2;
 
 	/* Setup Mailbox pointers */
-	((SLI2_SLIM_t *) phba->slim2p.virt)->un.slim.pcb.mailBoxSize =
-	    sizeof (MAILBOX_t);
-	offset =
-	    (uint8_t *) (&((SLI2_SLIM_t *) phba->slim2p.virt)->un.slim.mbx) -
-	    (uint8_t *) phba->slim2p.virt;
-	pdma_addr = phba->slim2p.phys + offset;
-	((SLI2_SLIM_t *) phba->slim2p.virt)->un.slim.pcb.mbAddrHigh =
-	    putPaddrHigh(pdma_addr);
-	((SLI2_SLIM_t *) phba->slim2p.virt)->un.slim.pcb.mbAddrLow =
-	    putPaddrLow(pdma_addr);
+	phba->slim2p->pcb.mailBoxSize = sizeof(MAILBOX_t);
+	offset = (uint8_t *)&phba->slim2p->mbx - (uint8_t *)phba->slim2p;
+	pdma_addr = phba->slim2p_mapping + offset;
+	phba->slim2p->pcb.mbAddrHigh = putPaddrHigh(pdma_addr);
+	phba->slim2p->pcb.mbAddrLow = putPaddrLow(pdma_addr);
 
-	
-	pci_read_config_dword(phba->pcidev, PCI_BASE_ADDRESS_0, &bar0_config_word);
-	pci_read_config_dword(phba->pcidev, PCI_BASE_ADDRESS_1, &bar1_config_word);
-	if (bar0_config_word & PCI_BASE_ADDRESS_MEM_TYPE_64) {
-		((SLI2_SLIM_t *) phba->slim2p.virt)->
-			un.slim.pcb.hgpAddrHigh = bar1_config_word;
-	} else {
-		((SLI2_SLIM_t *) phba->slim2p.virt)->
-			un.slim.pcb.hgpAddrHigh = 0;
+	/*
+	 * Setup Host Group ring pointer.
+	 *
+	 * For efficiency reasons, the ring get/put pointers can be
+	 * placed in adapter memory (SLIM) rather than in host memory.
+	 * This allows firmware to avoid PCI reads/writes when updating
+	 * and checking pointers.
+	 *
+	 * The firmware recognizes the use of SLIM memory by comparing
+	 * the address of the get/put pointers structure with that of
+	 * the SLIM BAR (BAR0).
+	 *
+	 * Caution: be sure to use the PCI config space value of BAR0/BAR1
+	 * (the hardware's view of the base address), not the OS's
+	 * value of pci_resource_start() as the OS value may be a cookie
+	 * for ioremap/iomap.
+	 */
+
+
+	pci_read_config_dword(phba->pcidev, PCI_BASE_ADDRESS_0, &bar_low);
+	pci_read_config_dword(phba->pcidev, PCI_BASE_ADDRESS_1, &bar_high);
+
+
+	/* mask off BAR0's flag bits 0 - 3 */
+	phba->slim2p->pcb.hgpAddrLow = (bar_low & PCI_BASE_ADDRESS_MEM_MASK) +
+					(SLIMOFF*sizeof(uint32_t));
+	if (bar_low & PCI_BASE_ADDRESS_MEM_TYPE_64)
+		phba->slim2p->pcb.hgpAddrHigh = bar_high;
+	else
+		phba->slim2p->pcb.hgpAddrHigh = 0;
+	/* write HGP data to SLIM at the required longword offset */
+	memset(&hgp, 0, sizeof(HGP));
+	to_slim = (uint8_t *)phba->MBslimaddr + (SLIMOFF*sizeof (uint32_t));
+	for (i=0; i < phba->sli.sliinit.num_rings; i++) {
+		lpfc_memcpy_to_slim(to_slim, &hgp, sizeof (HGP));
+		to_slim += sizeof (HGP);
 	}
-	bar0_config_word &= PCI_BASE_ADDRESS_MEM_MASK;
 
-	((SLI2_SLIM_t *) phba->slim2p.virt)->un.slim.pcb.hgpAddrLow = 
-		(uint32_t) (bar0_config_word
-			    + (SLIMOFF * sizeof (uint32_t)));
-	memset(&hgp, 0, sizeof (HGP));
-
-	/* write HGP data to SLIM */
-	to_slim = (uint8_t *) phba->MBslimaddr
-		+ (SLIMOFF * sizeof (uint32_t));
-	lpfc_memcpy_to_slim( to_slim,  &hgp, sizeof (HGP));
-	
-
-	/* Setup Port Group ring counters */
-	offset =
-	    (uint8_t *) (&((SLI2_SLIM_t *) phba->slim2p.virt)->un.slim.mbx.us.
-			 s2.port) - (uint8_t *) phba->slim2p.virt;
-	pdma_addr = phba->slim2p.phys + offset;
-	((SLI2_SLIM_t *) phba->slim2p.virt)->un.slim.pcb.pgpAddrHigh =
-	    putPaddrHigh(pdma_addr);
-	((SLI2_SLIM_t *) phba->slim2p.virt)->un.slim.pcb.pgpAddrLow =
-	    putPaddrLow(pdma_addr);
+	/* Setup Port Group ring pointer */
+	offset = (uint8_t *)&phba->slim2p->mbx.us.s2.port -
+		 (uint8_t *)phba->slim2p;
+	pdma_addr = phba->slim2p_mapping + offset;
+	phba->slim2p->pcb.pgpAddrHigh = putPaddrHigh(pdma_addr);
+	phba->slim2p->pcb.pgpAddrLow = putPaddrLow(pdma_addr);
 
 	/* Use callback routine to setp rings in the pcb */
-	hbainit = lpfc_config_pcb_setup(phba);
-	if (hbainit != 0)
+	lpfc_config_pcb_setup(phba);
+
+	/* special handling for LC HBAs */
+	if (lpfc_is_LC_HBA(phba->pcidev->device)) {
+		uint32_t hbainit[5];
+
+		lpfc_hba_init(phba, hbainit);
+
 		memcpy(&mb->un.varCfgPort.hbainit, hbainit, 20);
+	}
 
 	/* Swap PCB if needed */
-	lpfc_sli_pcimem_bcopy((uint32_t
-			      *) (&((SLI2_SLIM_t *) phba->slim2p.virt)->un.slim.
-				  pcb),
-			     (uint32_t
-			      *) (&((SLI2_SLIM_t *) phba->slim2p.virt)->un.slim.
-				  pcb), sizeof (PCB_t));
+	lpfc_sli_pcimem_bcopy((uint32_t *)&phba->slim2p->pcb,
+			      (uint32_t *)&phba->slim2p->pcb,
+			      sizeof (PCB_t));
 
-	/* Service Level Interface (SLI) 2 selected */
-	lpfc_printf_log(phba->brd_no, &lpfc_msgBlk0405,
-		       lpfc_mes0405, lpfc_msgBlk0405.msgPreambleStr);
-	return (0);
+	lpfc_printf_log(phba, KERN_INFO, LOG_INIT,
+		        "%d:0405 Service Level Interface (SLI) 2 selected\n",
+		        phba->brd_no);
 }
 
 void
-lpfc_mbox_put(lpfcHBA_t * phba, LPFC_MBOXQ_t * mbq)
-{				
-	LPFC_SLI_t *psli;
+lpfc_kill_board(struct lpfc_hba * phba, LPFC_MBOXQ_t * pmb)
+{
+	MAILBOX_t *mb = &pmb->mb;
+
+	memset(pmb, 0, sizeof(LPFC_MBOXQ_t));
+
+	mb->mbxCommand = MBX_KILL_BOARD;
+	mb->mbxOwner = OWN_HOST;
+	return;
+}
+
+void
+lpfc_mbox_put(struct lpfc_hba * phba, LPFC_MBOXQ_t * mbq)
+{
+	struct lpfc_sli *psli;
 
 	psli = &phba->sli;
 
@@ -713,14 +677,10 @@ lpfc_mbox_put(lpfcHBA_t * phba, LPFC_MBOXQ_t * mbq)
 }
 
 LPFC_MBOXQ_t *
-lpfc_mbox_get(lpfcHBA_t * phba)
+lpfc_mbox_get(struct lpfc_hba * phba)
 {
-	LPFC_MBOXQ_t *mbq;
-	LPFC_SLI_t *psli;
-	
-	mbq = 0;
-
-	psli = &phba->sli;
+	LPFC_MBOXQ_t *mbq = NULL;
+	struct lpfc_sli *psli = &phba->sli;
 
 	if (!list_empty(&psli->mboxq)) {
 		mbq = list_entry(psli->mboxq.next, LPFC_MBOXQ_t, list);
@@ -728,5 +688,21 @@ lpfc_mbox_get(lpfcHBA_t * phba)
 		psli->mboxq_cnt--;
 	}
 
-	return (mbq);
+	return mbq;
+}
+
+int
+lpfc_mbox_tmo_val(struct lpfc_hba *phba, int cmd)
+{
+	switch (cmd) {
+	case MBX_WRITE_NV:	/* 0x03 */
+	case MBX_UPDATE_CFG:	/* 0x1B */
+	case MBX_DOWN_LOAD:	/* 0x1C */
+	case MBX_DEL_LD_ENTRY:	/* 0x1D */
+	case MBX_LOAD_AREA:	/* 0x81 */
+	case MBX_FLASH_WR_ULA:  /* 0x98 */
+	case MBX_LOAD_EXP_ROM:	/* 0x9C */
+		return LPFC_MBOX_TMO_FLASH_CMD;
+	}
+	return LPFC_MBOX_TMO;
 }
